/************************************************************************
 *   IRC - Internet Relay Chat, m_core/m_clack.c
 *   This file is copyright (C) 2001 Andrew Suffield
 *                                    <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "ircd.h"
#include "numeric.h"
#include "irc_string.h"
#include "send.h"
#include "umodes.h"
#include "msg.h"
#include "handlers.h"
#include "client.h"
#include "burst.h"
#include "s_log.h"

#include <string.h>

int m_clack(struct Client *, struct Client *, int, char **);

struct Message clack_msgtab = {
  "CLACK", 0, 0, 0, MFLG_SLOW, 0,
  {m_unregistered, m_ignore, m_clack},
  NULL, NULL
};

/*
 * m_clack
 */
int
m_clack(struct Client *cptr, struct Client *sptr, int parc, char *parv[])
{
  if (!MyConnect(sptr))
    {
      sendto_ops_flag(UMODE_DEBUG, "WTF: Routed CLACK received from %s",
		      sptr->name);
      return -1;
    }

  if (!burst_in_progress)
    {
      sendto_ops_flag(UMODE_DEBUG, "WTF: CLACK received from %s, but nothing is bursting",
		      sptr->name);
      return -1;
    }

  if (sptr->serv->clacked)
    {
      sendto_ops_flag(UMODE_DEBUG, "WTF: %s CLACKed me multiple times",
		      sptr->name);
      return -1;
    }

  sptr->serv->clacked = 1;

  if (cptr != burst_in_progress)
    waiting_for_clacks--;

  if (waiting_for_clacks <= 0)	/* Everything locked, ready to go */
    {
      int need_to_burst = 0;

      /* OK, are we connecting at all? */
      if (local_burst)
	{
	  /* Are we the side which is, canonically, supposed to burst first? */
	  if (irccmp(me.name, burst_in_progress->name) > 0)
	    {
	      /* Burst if the other side has already clacked me, otherwise wait */
	      need_to_burst = burst_in_progress->serv->clacked;
	    }

	  /* Override that if this is a transfer clack */
	  if ((parc == 2 ) && !irccmp(parv[1], "transfer"))
	    need_to_burst = 1;
	}

      if (!already_clacked)
	{
	  sendto_one(burst_in_progress, ":%s CLACK", me.name);
	  already_clacked = 1;
	}

      if (need_to_burst)
	{
	  burst_to(burst_in_progress);

	  /* XXX - HACK!
	   * This indicates to m_burstdone when it gets called again that
	   * I'm already done
	   */
	  burst_in_progress->serv->clacked = 0;
	}
    }

  return 0;
}
